@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCmdIL.c,v 1.24.2.3 2001/10/08 15:50:24 dkf Exp $
+ * RCS: @(#) $Id: tclCmdIL.c,v 1.24.2.1 2001/04/03 22:54:36 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -72,13 +72,6 @@ typedef struct SortInfo {
 #define SORTMODE_COMMAND    3
 #define SORTMODE_DICTIONARY 4
 
-/*
- * Magic values for the index field of the SortInfo structure.
- * Note that the index "end-1" will be translated to SORTIDX_END-1, etc.
- */
-#define SORTIDX_NONE	-1		/* Not indexed; use whole value. */
-#define SORTIDX_END	-2		/* Indexed from end. */
-
 /*
  * Forward declarations for procedures defined in this file:
  */
@@ -569,17 +562,23 @@ InfoBodyCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
 
-    /* 
-     * Here we used to return procPtr->bodyPtr, except when the body was
-     * bytecompiled - in that case, the return was a copy of the body's
-     * string rep. In order to better isolate the implementation details
-     * of the compiler/engine subsystem, we now always return a copy of 
-     * the string rep. It is important to return a copy so that later 
-     * manipulations of the object do not invalidate the internal rep.
+    /*
+     * We should not return a bytecompiled body.  If it is precompiled,
+     * then the bodyPtr's string representation is bogus, since sources
+     * are not available.  If it was just a bytecompiled body, then it
+     * is likely to not be of any use to the caller, as it was compiled
+     * for a separate procedure context [Bug: 3412], and noone else can
+     * reasonably use it.
+     * In order to make sure that later manipulations of the object do not
+     * invalidate the internal representation, we make a copy of the string
+     * representation and return that one, instead.
      */
 
     bodyPtr = procPtr->bodyPtr;
-    resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
+    resultPtr = bodyPtr;
+    if (bodyPtr->typePtr == &tclByteCodeType) {
+	resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
+    }
     
     Tcl_SetObjResult(interp, resultPtr);
     return TCL_OK;
@@ -1507,19 +1506,19 @@ InfoProcsCmd(dummy, interp, objc, objv)
 	            || Tcl_StringMatch(cmdName, simplePattern)) {
 		cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
 
+		if (specificNsInPattern) {
+		    elemObjPtr = Tcl_NewObj();
+		    Tcl_GetCommandFullName(interp, (Tcl_Command) cmdPtr,
+		            elemObjPtr);
+		} else {
+		    elemObjPtr = Tcl_NewStringObj(cmdName, -1);
+		}
+
 		realCmdPtr = (Command *)
 		    TclGetOriginalCommand((Tcl_Command) cmdPtr);
 
 		if (TclIsProc(cmdPtr)
 		        || ((realCmdPtr != NULL) && TclIsProc(realCmdPtr))) {
-		    if (specificNsInPattern) {
-			elemObjPtr = Tcl_NewObj();
-			Tcl_GetCommandFullName(interp, (Tcl_Command) cmdPtr,
-				elemObjPtr);
-		    } else {
-			elemObjPtr = Tcl_NewStringObj(cmdName, -1);
-		    }
-
 		    Tcl_ListObjAppendElement(interp, listPtr, elemObjPtr);
 		}
 	    }
@@ -2520,7 +2519,7 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 
     sortInfo.isIncreasing = 1;
     sortInfo.sortMode = SORTMODE_ASCII;
-    sortInfo.index = SORTIDX_NONE;
+    sortInfo.index = -1;
     sortInfo.interp = interp;
     sortInfo.resultCode = TCL_OK;
     cmdPtr = NULL;
@@ -2561,8 +2560,8 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 			    -1);
 		    return TCL_ERROR;
 		}
-		if (TclGetIntForIndex(interp, objv[i+1], SORTIDX_END,
-			&sortInfo.index) != TCL_OK) {
+		if (TclGetIntForIndex(interp, objv[i+1], -2, &sortInfo.index)
+			!= TCL_OK) {
 		    return TCL_ERROR;
 		}
 		i++;
@@ -2815,20 +2814,20 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 
 	return order;
     }
-    if (infoPtr->index != SORTIDX_NONE) {
+    if (infoPtr->index != -1) {
 	/*
 	 * The "-index" option was specified.  Treat each object as a
 	 * list, extract the requested element from each list, and
-	 * compare the elements, not the lists.  "end"-relative indices
-	 * are signaled here with large negative values.
+	 * compare the elements, not the lists.  The special index "end"
+	 * is signaled here with a large negative index.
 	 */
 
 	if (Tcl_ListObjLength(infoPtr->interp, objPtr1, &listLen) != TCL_OK) {
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
 	}
-	if (infoPtr->index < SORTIDX_NONE) {
-	    index = listLen + infoPtr->index + 1;
+	if (infoPtr->index < -1) {
+	    index = listLen - 1;
 	} else {
 	    index = infoPtr->index;
 	}
@@ -2854,8 +2853,8 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
 	}
-	if (infoPtr->index < SORTIDX_NONE) {
-	    index = listLen + infoPtr->index + 1;
+	if (infoPtr->index < -1) {
+	    index = listLen - 1;
 	} else {
 	    index = infoPtr->index;
 	}
